package envoy.server.data;

import java.util.Date;
import java.util.Map;

import javax.persistence.*;

import envoy.data.MessageBuilder;

/**
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>GroupMessage.java</strong><br>
 * Created: <strong>18.04.2020</strong><br>
 * 
 * @author Maximilian K&auml;fer
 * @since Envoy Server Standalone v0.1-beta
 */
@Entity
public class GroupMessage extends Message {

	@ElementCollection
	private Map<Long, envoy.data.Message.MessageStatus> memberMessageStatus;

	@Column(name = "last_status_change_date")
	@Temporal(TemporalType.TIMESTAMP)
	protected Date lastStatusChangeDate;

	/**
	 * The constructor for a database object.
	 *
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public GroupMessage() {}

	/**
	 * Constructs a database groupMessage from a common groupMessage.
	 *
	 * @param groupMessage         the {@link envoy.data.GroupMessage} to convert
	 *                             into a
	 *                             database {@link GroupMessage}
	 * @param lastStatusChangeDate the {@link Date} to set
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public GroupMessage(envoy.data.GroupMessage groupMessage, Date lastStatusChangeDate) {
		super(groupMessage);
		memberMessageStatus	= groupMessage.getMemberStatuses();
		this.lastStatusChangeDate	= lastStatusChangeDate;
	}

	/**
	 * Converts this groupMessage into an instance of
	 * {@link envoy.data.GroupMessage}.
	 *
	 * @return a {@link envoy.data.GroupMessage} containing the same values as this
	 *         groupMessage
	 * @since Envoy Server Standalone v0.1-beta
	 */
	@Override
	public envoy.data.GroupMessage toCommon() {
		// TODO: Attachment
		envoy.data.GroupMessage groupMessage = new MessageBuilder(sender.getID(), recipient.getID(), id).setDate(creationDate)
			.setForwarded(forwarded)
			.setStatus(status)
			.setText(text)
			.buildGroupMessage((envoy.data.Group) recipient.toCommon(), memberMessageStatus);
		groupMessage.setReceivedDate(receivedDate);
		groupMessage.setReadDate(readDate);
		return groupMessage;
	}

	/**
	 * @return the memberMessageStatus
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public Map<Long, envoy.data.Message.MessageStatus> getMemberMessageStatus() { return memberMessageStatus; }

	/**
	 * @param memberMessageStatus the memberMessageStatus to set
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public void setMemberMessageStatus(Map<Long, envoy.data.Message.MessageStatus> memberMessageStatus) {
		this.memberMessageStatus = memberMessageStatus;
	}

	/**
	 * @return the date at which one of the member statuses changed last
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public Date getLastStatusChangeDate() { return lastStatusChangeDate; }

	/**
	 * @param date the date to set
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public void setLastStatusChangeDate(Date date) { lastStatusChangeDate = date; }
}
