package envoy.server.processors;

import java.io.IOException;
import java.util.Date;
import java.util.logging.Logger;

import javax.persistence.EntityExistsException;

import envoy.data.GroupMessage;
import envoy.data.Message.MessageStatus;
import envoy.server.data.PersistenceManager;
import envoy.server.net.ConnectionManager;
import envoy.server.net.ObjectWriteProxy;
import envoy.util.EnvoyLog;

/**
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>GroupMessageProcessor.java</strong><br>
 * Created: <strong>18.04.2020</strong><br>
 * 
 * @author Maximilian K&auml;fer
 * @since Envoy Server Standalone v0.1-beta
 */
public class GroupMessageProcessor implements ObjectProcessor<GroupMessage> {

	private static final Logger logger = EnvoyLog.getLogger(GroupCreationProcessor.class);

	@Override
	public void process(GroupMessage groupMessage, long socketID, ObjectWriteProxy writeProxy) {
		groupMessage.nextStatus();
		ConnectionManager connectionManager = ConnectionManager.getInstance();

		final var members = PersistenceManager.getInstance().getGroupByID(groupMessage.getRecipientID()).getContacts();
		groupMessage.getMemberStatuses().replaceAll((id, oldStatus) -> MessageStatus.SENT);
		members.forEach(user -> setMemberStatus(connectionManager, groupMessage, user.getID()));

		// Checks if all memberMessageStatuses are RECEIVED and if so sets the
		// groupMessage Status to RECEIVED.

		members.forEach(user -> { sendToMember(connectionManager, groupMessage, user.getID(), writeProxy); });

		try {
			PersistenceManager.getInstance().addMessage(new envoy.server.data.GroupMessage(groupMessage, new Date()));
		} catch (EntityExistsException e) {
			logger.warning("Received a groupMessage with an ID that already exists");
		}
	}

	private void sendToMember(ConnectionManager connectionManager, GroupMessage groupMessage, long memberID, ObjectWriteProxy writeProxy) {
		if (connectionManager.isOnline(memberID)) try {
			// If recipient is online, send the groupMessage directly
			writeProxy.write(connectionManager.getSocketID(memberID), groupMessage);
		} catch (IOException e) {
			logger.warning("Recipient online. Failed to send message" + groupMessage.getID());
			e.printStackTrace();
		}
	}

	private void setMemberStatus(ConnectionManager connectionManager, GroupMessage groupMessage, long memberID) {
		if (connectionManager.isOnline(memberID))
			// Update the message status of the member to RECEIVED
			groupMessage.getMemberStatuses().replace(memberID, MessageStatus.RECEIVED);
	}

	@Override
	public Class<GroupMessage> getInputClass() { return GroupMessage.class; }
}
