package envoy.server.processors;

import java.io.IOException;

import envoy.event.GroupResize;
import envoy.server.data.Contact;
import envoy.server.data.PersistenceManager;
import envoy.server.net.ConnectionManager;
import envoy.server.net.ObjectWriteProxy;

/**
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>GroupResizeProcessor.java</strong><br>
 * Created: <strong>03.04.2020</strong><br>
 *
 * @author Maximilian K&auml;fer
 * @since Envoy Server Standalone v0.1-beta
 */
public class GroupResizeProcessor implements ObjectProcessor<GroupResize> {

	private static final PersistenceManager	persistenceManager	= PersistenceManager.getInstance();
	private static final ConnectionManager	connectionManager	= ConnectionManager.getInstance();

	@Override
	public void process(GroupResize groupResize, long socketID, ObjectWriteProxy writeProxy) throws IOException {

		// Acquire the group to resize from the database
		var group = persistenceManager.getGroupByID(groupResize.getGroupID());

		// Perform the desired operation
		switch (groupResize.getOperation()) {
			case ADD:
				group.getContacts().add(persistenceManager.getUserByID(groupResize.get().getID()));
				break;
			case REMOVE:
				group.getContacts().remove(persistenceManager.getUserByID(groupResize.get().getID()));
				break;
		}

		// Update the group in the database
		persistenceManager.updateContact(group);

		// Send the updated group to all of its members
		var commonGroup = group.toCommon();
		group.getContacts()
			.stream()
			.map(Contact::getID)
			.filter(connectionManager::isOnline)
			.map(connectionManager::getSocketID)
			.forEach(memberSocketID -> {
				try {
					writeProxy.write(memberSocketID, commonGroup);
				} catch (IOException e) {
					e.printStackTrace();
				}
			});
	}

	@Override
	public Class<GroupResize> getInputClass() { return GroupResize.class; }
}
