package envoy.server.processors;

import java.io.IOException;

import envoy.data.Message.MessageStatus;
import envoy.event.MessageStatusChange;
import envoy.exception.EnvoyException;
import envoy.server.data.PersistenceManager;
import envoy.server.net.ConnectionManager;
import envoy.server.net.ObjectWriteProxy;

/**
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>MessageStatusChangeProcessor.java</strong><br>
 * Created: <strong>10 Jan 2020</strong><br>
 *
 * @author Leon Hofmeister
 * @since Envoy Server Standalone v0.1-alpha
 */
public class MessageStatusChangeProcessor implements ObjectProcessor<MessageStatusChange> {

	private final PersistenceManager	persistenceManager	= PersistenceManager.getInstance();
	private final ConnectionManager		connectionManager	= ConnectionManager.getInstance();

	@Override
	public void process(MessageStatusChange statusChange, long socketID, ObjectWriteProxy writeProxy) throws IOException {
		// Any other status than READ is not supposed to be sent to the server
		if (statusChange.get() != MessageStatus.READ) throw new IOException(new EnvoyException(statusChange + " has an invalid status"));

		final var msg = persistenceManager.getMessageByID(statusChange.getID());
		msg.read();
		persistenceManager.updateMessage(msg);

		// Notifies the sender of the message about the status-update to READ
		final long senderID = msg.getSender().getID();
		if (connectionManager.isOnline(senderID)) writeProxy.write(connectionManager.getSocketID(senderID), statusChange);
	}

	@Override
	public Class<MessageStatusChange> getInputClass() { return MessageStatusChange.class; }
}
