package envoy.server.processors;

import java.io.IOException;

import envoy.event.NameChange;
import envoy.server.data.Contact;
import envoy.server.data.PersistenceManager;
import envoy.server.data.User;
import envoy.server.net.ConnectionManager;
import envoy.server.net.ObjectWriteProxy;

/**
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>NameChangeProcessor.java</strong><br>
 * Created: <strong>26 Mar 2020</strong><br>
 *
 * @author Leon Hofmeister
 * @since Envoy Server Standalone v0.1-beta
 */
public class NameChangeProcessor implements ObjectProcessor<NameChange> {

	@Override
	public void process(NameChange nameChange, long socketID, ObjectWriteProxy writeProxy) throws IOException {
		PersistenceManager	persistenceManager	= PersistenceManager.getInstance();
		ConnectionManager	connectionManager	= ConnectionManager.getInstance();
		Contact				toUpdate			= persistenceManager.getContactByID(nameChange.getID());
		toUpdate.setName(nameChange.get());
		persistenceManager.updateContact(toUpdate);

		// Notify online contacts of the name change
		toUpdate.getContacts().stream().filter(User.class::isInstance).map(Contact::getID).filter(connectionManager::isOnline).forEach(userID -> {
			try {
				writeProxy.write(userID, nameChange);
			} catch (IOException e) {
				e.printStackTrace();
			}
		});
	}

	@Override
	public Class<NameChange> getInputClass() { return NameChange.class; }
}
